
/* 
   Project:     SOFTSUSY 1.9
   File:        main.cpp
   Author:      Ben Allanach 
   Manual:      B.C. Allanach,hep-ph/0104145, Comp. Phys. Comm. 143 (2002) 305 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html
   Description: main calling program example: performs a scan of tan beta 
   (starting at mSUGRA point SPS1a) and prints out Higgs masses as a result in
   the format
       <tan beta>      <mh0>     <mA0>    <mH0>     <mH+/->
*/

#include "iostream.h"
#include "complex.h"
#include "def.h"
#include "linalg.h"
#include "lowe.h"
#include "rge.h"
#include "softsusy.h"
#include "softpars.h"
#include "susy.h"
#include "utils.h"
#include "numerics.h"

/// global variable declaration
// no quark mixing, and no verbose output
int MIXING = -1, PRINTOUT = 0;
// fractional accuracy required
double TOLERANCE = 1.0e-3;
// decay constant of muon
double GMU = 1.16637e-5; 
/// end of global variable declaration

int main() {
 // Sets format of output: 4 decimal places
  outputCharacteristics(4);

  cerr << "SOFTSUSY" << VERSION << " test program, Ben Allanach 2002\n";
  cerr << "If you use SOFTSUSY, please refer to B.C. Allanach, \n";
  cerr << " Comput. Phys. Commun. 143 (2002) 305, hep-ph/0104145\n\n";

  // Parameters used: mSUGRA parameters
  double m12 = 250., a0 = -100., mGutGuess = 2.0e16, tanb = 10.0, m0 = 100.;
  int sgnMu = 1;      // sign of mu parameter 
  int numPoints = 10; // number of scan points

  QedQcd oneset;      // See "lowe.h" for default definitions of SM parameters
  oneset.toMz();      // Runs SM fermion masses to MZ

  cerr << "Low energy data in SOFTSUSY: MIXING=" << MIXING << " TOLERANCE=" 
       << TOLERANCE << endl << oneset << endl;

  int i; 
  // Set limits of tan beta scan
  double startTanb = 3.0, endTanb = 50.0;
  // Cycle through different points in the scan
  for (i = 0; i<=numPoints; i++) {

    tanb = (endTanb - startTanb) / double(numPoints) * double(i) +
      startTanb; // set tan beta ready for the scan.

    // Preparation for calculation: set up object and input parameters
    MssmSoftsusy r; 
    DoubleVector pars(3); 
    pars(1) = m0; pars(2) = m12; pars(3) = a0;
    bool uni = true; // Unify gauge couplings
    
    // Calculate the spectrum
    r.lowOrg(sugraBcs, mGutGuess, pars, sgnMu, tanb, oneset, uni);
    
    // check the point in question is problem free: if so print the output
    if (!r.displayProblem().test()) 
      cout << tanb << " " 
	   << r.displayPhys().mhiggs(1) << " " 
	   << r.displayPhys().mhiggs(2) << " " 
	   << r.displayPhys().mhiggs(3) << " " 
	   << r.displayPhys().mhiggs(4) << endl;
    else
      // print out what the problem(s) is(are)
      cout << tanb << " " << r.displayProblem() << endl;
  }
}
