
/** \file softpoint.h
   - Project:     SOFTSUSY 
   - Author:      Ben Allanach 
   - Manual:      hep-ph/0104145, Comp. Phys. Comm. 143 (2002) 305 
   - Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html
   - Description: main calling program: command line interface

   $Log: softpoint.h,v $
   Revision 1.8  2003/10/24 16:30:26  allanach
   Deleted old higgsVevMs variable, using running variable instead

   Revision 1.7  2003/08/19 14:26:22  allanach
   Changing lowOrg to be more sensible about gauge unification. Should now be
   called with POSITIVE mgut and a flag for gauge unification.

   Revision 1.6  2003/08/19 14:02:03  allanach
   Can now use "unified" to flag gauge unification

   Revision 1.5  2003/07/24 14:55:28  allanach
   Implemented les Houches input and output properly in the usual command-line
   interface

   Revision 1.4  2003/07/21 14:00:18  allanach
   MZ fully implemented as an input now. Kept MZ as the central PDG 2002 value,
   for defaults etc

   Revision 1.3  2003/05/22 12:45:41  allanach
   Fortran version of 2-loop Higgs corrections included with corrections:
   alpha_s (alpha_t + alpha_b) + alpha_t^2 + alpha_b^2 + alpha_t alpha_b

   Revision 1.2  2003/05/20 15:19:40  allanach
   doxygen comment style implemented

   Revision 1.1  2003/04/02 09:19:42  allanach
   Relegated MssmSoftsusy2 definition (and other headers) here

*/

#include <iostream>
#include <sstream>
#include <string>
using namespace std;
#include <mycomplex.h>
#include <def.h>
#include <linalg.h>
#include <lowe.h>
#include <rge.h>
#include <softsusy.h>
#include <softpars.h>
#include <susy.h>
#include <utils.h>
#include <numerics.h>
#include <twoloophiggs.h>

/// Does the user require gauge unification or not -- gaugeUnification changed
/// to be correct value
inline double mgutCheck(char * a, bool & gaugeUnification) {
  gaugeUnification = false;
  if (!strcmp(a, "?") || !strcmp(a,"unified")) {
    gaugeUnification = true;
    return 2.0e16;
  }
  else return atof(a);
};

/// Incorrect input: gives advice on how to supply it
void errorCall();

/// Just like the usual MSSM SOFTSUSY class, but
/// enforces a different REWSB condition - given by mu(MS) and MA(pole)
/// conditions, instead of specifying mh1(MGUT) and mh2(MGUT)
class MssmSoftsusy2: public MssmSoftsusy {
private:
public:
  double mAcond, muCond; ///< user set conditions on mA and mu at M_SUSY
  /// Just sets the mu parameter to what the user wanted
  virtual void rewsbTreeLevel(int sgnMu) {
    setSusyMu(muCond);
  };
  /// "Inverts" the relation between SUSY breaking parameters and mA in order
  /// that mA(pole) is set by the user constraint: mH1^2 and mH2^2 are set in
  /// order to be consistent with mu and mA given.
  virtual void rewsb(int sgnMu, double mt) {
    setSusyMu(muCond);
    calcDrBarPars();
    double sinthDRbarMS = calcSinthdrbar();
    double tanb = displayTanb(), beta = atan(tanb);
    double mz = displayMz();
    double pizztMS = piZZT(mz, displayMu());
    doTadpoles(mt, sinthDRbarMS);

    setHvev(getVev()); 
    double piaa = piAA(mAcond, displayMu());

    double mAsq =  sqr(displayDrBarPars().mhiggs(2));

    double gstrong = displayGaugeCoupling(3), 
      rmtsq = sqr(displayDrBarPars().mt), scalesq = sqr(displayMu()), 
      vev2 = sqr(displayHvev()), tbeta = displayTanb(), 
      amu = -displaySusyMu(), mgsq = sqr(displayGaugino()(3));
    // two-loop Higgs corrections: alpha_s alpha_b
    double sxb = sin(displayDrBarPars().thetab), 
      cxb = cos(displayDrBarPars().thetab);
    double msb1sq = sqr(displayDrBarPars().md(1, 3)), 
      msb2sq = sqr(displayDrBarPars().md(2, 3));
    double cotbeta = 1.0 / tbeta;
    double rmbsq = sqr(displayDrBarPars().mb);

    double sxt = sin(displayDrBarPars().thetat), 
      cxt = cos(displayDrBarPars().thetat);
    double mst1sq = sqr(displayDrBarPars().mu(1, 3)), 
      mst2sq = sqr(displayDrBarPars().mu(2, 3));
    double p2s = 0., p2w = 0., p2b = 0.;
    dszodd_(&rmtsq, &mgsq, &mst1sq, &mst2sq, &sxt, &cxt, &scalesq, &amu,
	    &tbeta, &vev2, &gstrong, &p2s); 
    ddsodd_(&rmtsq, &rmbsq, &mAsq, &mst1sq, &mst2sq, &msb1sq, &msb2sq, 
	    &sxt, &cxt, &sxb, &cxb, &scalesq, &amu, &tanb, &vev2, 
	    &p2w);
    dszodd_(&rmbsq, &mgsq, &msb1sq, &msb2sq, &sxb, &cxb, &scalesq, &amu,
	    &cotbeta, &vev2, &gstrong, &p2b); 

    double dMA = p2s + p2b + p2w;

    double newMh1sq, newMh2sq;
    newMh1sq = sqr(sin(beta)) * (sqr(mAcond) + piaa + sqr(mz) + pizztMS - dMA) 
      - (sqr(displaySusyMu()) + 0.5 * sqr(mz) + 0.5 * pizztMS) +
      displayTadpole1Ms() * (1.0 - sqr(sqr(sin(beta)))) -
      displayTadpole2Ms() * sqr(sin(beta)) * sqr(cos(beta));

    newMh2sq = sqr(cos(beta)) * (sqr(mAcond) + piaa + sqr(mz) + pizztMS - dMA) 
      - (sqr(displaySusyMu()) + 0.5 * sqr(mz) + 0.5 * pizztMS) -
      displayTadpole1Ms() * sqr(sin(beta)) * sqr(cos(beta)) +
      displayTadpole2Ms() * (1.0 - sqr(sqr(cos(beta))));

    setMh1Squared(newMh1sq);
    setMh2Squared(newMh2sq);

    double m3sqnew;
    if (rewsbM3sq(displaySusyMu(), m3sqnew) == 0) flagB(false);
    else flagB(true);
    setM3Squared(m3sqnew);
  
    if ((displayMh1Squared() + 2.0 * sqr(displaySusyMu()) +
       displayMh2Squared() - 2.0 * fabs(displayM3Squared())) < 0.0 )
    flagHiggsufb(true);
  else 
    flagHiggsufb(false);
  }
};


